# -*- coding: utf-8 -*-
# Copyright 2016 Camptocamp SA
# License LGPL-3.0 or later (https://www.gnu.org/licenses/lgpl-3.0.en.html)

from io import BytesIO

import pytest

from anthem.exceptions import AnthemError
import anthem.cli
from anthem.lyrics.loaders import load_csv, load_csv_stream

csv_partner = (b"id,name,street,city\n"
               b"__test__.partner1,Partner 1,Street 1,City 1\n"
               b"__test__.partner2,Partner 2,Street 2,City 2\n"
               )


def test_load_csv_stream_model():
    csv_stream = BytesIO()
    csv_stream.write(csv_partner)
    csv_stream.seek(0)
    with anthem.cli.Context(None, anthem.cli.Options(test_mode=True)) as ctx:
        load_csv_stream(ctx, ctx.env['res.partner'], csv_stream, delimiter=',')
        partner1 = ctx.env.ref('__test__.partner1', raise_if_not_found=False)
        assert partner1
        assert partner1.name == 'Partner 1'
        partner2 = ctx.env.ref('__test__.partner2', raise_if_not_found=False)
        assert partner2
        assert partner2.name == 'Partner 2'


def test_load_csv_file_model(tmpdir):
    csvfile = tmpdir.mkdir("files").join("res.partner.csv")
    csvfile.write(csv_partner)
    with anthem.cli.Context(None, anthem.cli.Options(test_mode=True)) as ctx:
        load_csv(ctx, ctx.env['res.partner'], csvfile.strpath, delimiter=',')
        partner1 = ctx.env.ref('__test__.partner1', raise_if_not_found=False)
        assert partner1
        assert partner1.name == 'Partner 1'
        partner2 = ctx.env.ref('__test__.partner2', raise_if_not_found=False)
        assert partner2
        assert partner2.name == 'Partner 2'


def test_load_csv_stream_model_string():
    """ Pass string instead of model to load_csv_stream """
    csv_stream = BytesIO()
    csv_stream.write(csv_partner)
    csv_stream.seek(0)
    with anthem.cli.Context(None, anthem.cli.Options(test_mode=True)) as ctx:
        load_csv_stream(ctx, 'res.partner', csv_stream, delimiter=',')
        partner1 = ctx.env.ref('__test__.partner1', raise_if_not_found=False)
        assert partner1
        assert partner1.name == 'Partner 1'
        partner2 = ctx.env.ref('__test__.partner2', raise_if_not_found=False)
        assert partner2
        assert partner2.name == 'Partner 2'


def test_load_csv_file_model_string(tmpdir):
    csvfile = tmpdir.mkdir("files").join("res.partner.csv")
    csvfile.write(csv_partner)
    with anthem.cli.Context(None, anthem.cli.Options(test_mode=True)) as ctx:
        load_csv(ctx, 'res.partner', csvfile.strpath, delimiter=',')
        partner1 = ctx.env.ref('__test__.partner1', raise_if_not_found=False)
        assert partner1
        assert partner1.name == 'Partner 1'
        partner2 = ctx.env.ref('__test__.partner2', raise_if_not_found=False)
        assert partner2
        assert partner2.name == 'Partner 2'


def test_load_erroneous_csv():
    err_csv = (b"id,name,category_id/id\n"
               b"__test__.partner_fail,Test, xmlid_not_found\n")
    csv_stream = BytesIO()
    csv_stream.write(err_csv)
    csv_stream.seek(0)
    with anthem.cli.Context(None, anthem.cli.Options(test_mode=True)) as ctx:
        with pytest.raises(AnthemError):
            load_csv_stream(ctx, 'res.partner', csv_stream, delimiter=',')



# -*- coding: utf-8 -*-
# Copyright 2016 Camptocamp SA
# License LGPL-3.0 or later (https://www.gnu.org/licenses/lgpl-3.0.en.html)

import anthem.cli
from anthem.lyrics.records import add_xmlid, create_or_update


def test_add_xmlid():
    with anthem.cli.Context(None, anthem.cli.Options(test_mode=True)) as ctx:
        env = ctx.env
        record = env['res.partner'].create({'name': 'test'})
        assert not record.get_metadata()[0]['xmlid']
        ref1 = add_xmlid(ctx, record, 'test.add_xmlid')
        assert record.get_metadata()[0]['xmlid'] == 'test.add_xmlid'
        ref2 = add_xmlid(ctx, record, 'test.add_xmlid')
        assert ref1 == ref2


def test_create_or_update():
    with anthem.cli.Context(None, anthem.cli.Options(test_mode=True)) as ctx:
        record = create_or_update(ctx, 'res.partner', 'test.upsert',
                                  {'name': 'test'})
        assert record.name == 'test'
        assert record._name == 'res.partner'
        record2 = create_or_update(ctx, 'res.partner', 'test.upsert',
                                   {'name': 'test2'})
        assert record2 == record
        assert record2.name == 'test2'



# -*- coding: utf-8 -*-
# Copyright 2016 Camptocamp SA
# License LGPL-3.0 or later (https://www.gnu.org/licenses/lgpl-3.0.en.html)

from pkg_resources import get_distribution, DistributionNotFound

try:
    __version__ = get_distribution(__name__).version
except DistributionNotFound:
    # package is not installed
    pass

# publish the decorator so we can use 'anthem.log'
from .output import log  # noqa



# -*- coding: utf-8 -*-
# Copyright 2016 Camptocamp SA
# License LGPL-3.0 or later (https://www.gnu.org/licenses/lgpl-3.0.en.html)

from __future__ import print_function

import os

from distutils.version import LooseVersion
from invoke import task, Collection

ns = Collection()
tests = Collection('tests')
ns.add_collection(tests)

ODOO_URL = 'https://github.com/odoo/odoo/archive/{}.tar.gz'


def is_below_odoo_10(version):
    return LooseVersion(version) < LooseVersion('10.0')


def dbname(version):
    return 'anthem-test-db-{}'.format(version.replace('.', '_'))


def assert_version(version):
    assert version in ('11.0', '10.0', '9.0')


@task
def tests_prepare(ctx, version):
    assert_version(version)
    if not os.path.exists('odoo-{}'.format(version)):
        url = ODOO_URL.format(version)
        print('Getting {}'.format(url))
        ctx.run('wget -nv -c -O odoo.tar.gz {}'.format(url))
        ctx.run('tar xfz odoo.tar.gz')
        ctx.run('rm -f odoo.tar.gz')
    print('Installing odoo, now')
    ctx.run('pip install -r odoo-{}/requirements.txt -q'.format(version))
    ctx.run('pip install -e odoo-{} -q'.format(version))


@task
def tests_createdb(ctx, version):
    assert_version(version)
    db = dbname(version)
    print('Installing database {}'.format(db))
    if is_below_odoo_10(version):
        ctx.run('odoo.py -d {} --workers=0 --log-level=critical '
                '--stop-after-init'.format(db))
    else:
        ctx.run('odoo -d {} --workers=0 --log-level=critical '
                '--stop-after-init'.format(db))


@task
def tests_dropdb(ctx, version):
    assert_version(version)
    print('Dropping the database')
    try:
        if is_below_odoo_10(version):
            import openerp as odoo
        else:
            import odoo  # noqa
        odoo.tools.config.parse_config(None)
        odoo.service.db.exp_drop(dbname(version))
    except ImportError:
        print('Could not import odoo')
        exit(1)


@task
def tests_prepare_config(ctx, version, source, target):
    assert_version(version)
    assert source and target
    with open(source, 'rU') as source_file:
        config_content = source_file.readlines()

    for idx, line in enumerate(config_content):
        if line.startswith('db_name'):
            config_content[idx] = 'db_name = {}\n'.format(dbname(version))

    with open(target, 'w') as config_file:
        for line in config_content:
            config_file.write(line)


@task(default=True)
def tests_prepare_version(ctx, version):
    tests_prepare(ctx, version)
    config_file = '/tmp/test-anthem-config-%s.cfg' % version
    tests_prepare_config(ctx, version, 'tests/config/odoo.cfg', config_file)
    tests_createdb(ctx, version)


tests.add_task(tests_prepare_version, 'prepare-version')
tests.add_task(tests_createdb, 'createdb')
tests.add_task(tests_dropdb, 'dropdb')
tests.add_task(tests_prepare, 'prepare')
tests.add_task(tests_prepare_config, 'prepare-config')



# -*- coding: utf-8 -*-
# Copyright 2016 Camptocamp SA
# License LGPL-3.0 or later (https://www.gnu.org/licenses/lgpl-3.0.en.html)
import subprocess
import pytest


def test_no_params_prints_usage():
    with pytest.raises(subprocess.CalledProcessError) as e:
        subprocess.check_output(['anthem'], stderr=subprocess.STDOUT)
    assert "usage" in e.value.output.decode()
    assert e.value.returncode == 2


def test_one_param_calls_song():
    result = subprocess.check_output(['anthem', 'anthem.sample_songs::empty'])
    assert result.decode() == "I am empty\n"



# -*- coding: utf-8 -*-
# Copyright 2016 Camptocamp SA
# License LGPL-3.0 or later (https://www.gnu.org/licenses/lgpl-3.0.en.html)

import pytest
import anthem.cli
from anthem.lyrics.modules import uninstall
from anthem.exceptions import AnthemError


def test_uninstall_1():
    with anthem.cli.Context(None, anthem.cli.Options(test_mode=True)) as ctx:
        with pytest.raises(AnthemError) as excinfo:
            uninstall(ctx, [])
        excinfo.match(r'You have to provide a list of module.*')


def test_uninstall_2():
    with anthem.cli.Context(None, anthem.cli.Options(test_mode=True)) as ctx:
        ctx.env['ir.module.module'].search(
            [('name', '=', 'lunch')]).button_immediate_install()
        uninstall(ctx, ['lunch'])
